% EGEEfem - A simple FEM Program
%
% To solve systems f = [K]u + [C]udot + [M]udotdot
%
% (c) Derek Elsworth - elsworth@psu.edu
%
% Version 2005.3 (9.13.2005)
% Updated for poromechanics 2014.1 (4.3.2014)
% -------------------------------------
%
% Read mesh data
%
    in_data = fem_input_25_2
%
% Define global system variables
%
    nodes           = in_data.control(1)
    numel           = in_data.control(2)
    nodaldof        = in_data.control(3)
    nodesperelement = in_data.control(4)
    numat           = in_data.control(5)
    ndimensions     = in_data.control(6)
    numeq           = nodes*nodaldof
    totalelementdof = nodaldof*nodesperelement
%
% Zero global system matrices
%
    Kg  = zeros(numeq,numeq);
    Cg  = zeros(numeq,numeq);
    Mg  = zeros(numeq,numeq);
    f   = zeros(numeq,1);
    u   = zeros(numeq,1);
    ud  = zeros(numeq,1);
    udd = zeros(numeq,1);

%
% Assemble Global Stiffness Matrix Kg[]
    for iel=1:numel   % Loop over all elements
%----------------------------------------------- 1. Localize element variables
         [local_data] = localize(iel,in_data);

    '++++++++ Localizing Data for'
            ElementNo = iel
            MaterialArray     = local_data.mater
            CoordinateArray   = local_data.coords
            dofAddressArray   = local_data.dofs
           
%----------------------------------------------- 2. Send local variables to form local stiffness matrix [Kl]
        MaterialType = in_data.EL(iel,2)
        ElementType  = in_data.mater(MaterialType,1)
         
         if (ElementType == 11) [Kl,Cl,Ml] = elmt11 (local_data); end  % 2-node 1-D element (bar) diffusion 
         if (ElementType == 12) [Kl,Cl,Ml] = elmt12 (local_data); end  % 2-node 1-D element (bar) diffusion-advection 
%        if (ElementType == 13) [Kl,Cl,Ml] = elmt13 (local_data); end  % 2-node 1-D element (bar) Navier-Stokes flow
         if (ElementType == 14) [Kl,Cl,Ml] = elmt14 (local_data); end  % 2-node 1-D element (bar) solid mechanics
         
         if (ElementType == 21) [Kl,Cl,Ml] = elmt21 (local_data); end  % 3-node 2-D element (triangle) diffusion
         if (ElementType == 22) [Kl,Cl,Ml] = elmt22 (local_data); end  % 3-node 2-D element (triangle) advection-diffusion
         if (ElementType == 23) [Kl,Cl,Ml] = elmt23 (local_data); end  % 3-node 2-D element (triangle) Navier-Stokes flow
         if (ElementType == 24) [Kl,Cl,Ml] = elmt24 (local_data); end  % 3-node 2-D element (triangle) solid mechanics 
         if (ElementType == 25) [Kl,Cl,Ml] = elmt25 (local_data); end  % 3-node 2-D element (triangle) poromechanics 

         '++++++++ Determining local stiffness matrix for '
            ElementNo = iel
            LocalStiffness = Kl
            LocalDamping   = Cl
            LocalMass      = Ml
 
%----------------------------------------------- 3. Assemble global stiffness matrix [Kg] from local stiffness matrix [Kl] etc
         [Kg] = assemble (Kg, Kl, local_data, totalelementdof);
         [Cg] = assemble (Cg, Cl, local_data, totalelementdof);
         [Mg] = assemble (Mg, Ml, local_data, totalelementdof);         
         
    '++++++++ Determining global stiffness matrix for '
            ElementNo = iel
            GlobalStiffness = Kg
            GlobalDamping   = Cg
            GlobalMass      = Mg
            
    end
%----------------------------------------------- 4. Assemble load and displacement vectors
        [f] = vectorstretch (in_data.LOAD, nodes, 1+nodaldof)  % convert force bc's to 1-D array
      [con] = vectorstretch (in_data.CON,  nodes, 1+nodaldof)  % convert constraints to 1-D array
         u  = f.* (con)                                        % convert to global displacement vector u
         f  = f.* (1-con)                                      % convert to global load vector f
         f  = f - Kg*u                                         % modify load vector for prescribed displacements

%----------------------------------------------- 5. Rearrange system of equations
        initialdisplacement = 0.0                              % iniital conditions 
        Ktemp = Kg;  Ctemp = Cg; Mtemp = Mg; ftemp = f;        % set temporary matrices for Kg...
        ftemp = f;   utemp = ones(numeq,1)*initialdisplacement;% set temporary vectors for f...
    for ieq = numeq: -1: 1
        if (con(ieq,1) == 1) Ktemp(:,ieq)=[]; Ktemp(ieq,:)=[]; % reorder stiffness matrix
                             Ctemp(:,ieq)=[]; Ctemp(ieq,:)=[]; % reorder damping matrix
                             Mtemp(:,ieq)=[]; Mtemp(ieq,:)=[]; % reorder mass matrix
                             ftemp(ieq,:)=[]; utemp(ieq,:)=[]; % reorder load vector
        end
    end

%----------------------------------------------- 6. Solve system of equations
        time = 0;   dt = .0001;    solswitch = 4                  % (1) Static; (2) Implicit; (3) Explicit; (4) Poromechanis (Impl)
        
        if (solswitch == 1)  utemp = Ktemp\ftemp               % (1) Static
        end
        
        if (solswitch == 2)                                    % (2) Dynamic Implicit (highest order term velocity)
            for itime = 1:30                   
                time  = time + dt
                utemp = (Ktemp + (1/dt)*Ctemp)\(ftemp + (1/dt)*(Ctemp*utemp))
            end
        end

        if (solswitch == 3)                                    % (3) Dynamic Explicit (highest order term velocity)
            for itime = 1:30       
                time  = time + dt
                utemp = utemp + Ctemp\(ftemp - Ktemp*utemp)*dt
            end
        end
        
        if (solswitch == 4)                                    % (4) Poromechanics - Dynamic Implicit
            ftemp = ftemp/dt
            for itime = 1:3                   
                time  = time + dt
                utemp = (Ktemp + (1/dt)*Ctemp)\(ftemp + (1/dt)*(Ctemp*utemp))
                ftemp = ftemp*0.0
            end
        end

%----------------------------------------------- 7. Resubstitute for unknowns

icount=0;                                                   % reconstitute displacement solution vector u
    for ieq = 1:numeq
        if (con(ieq,1) == 0) icount = icount + 1; u(ieq,1) = utemp(icount,1);
        end
    end

    u                                                            % output displacements
    f=Kg*u                                                       % evaluate complementary boundary condisons

    '************ End of Execution ******************* '    